% Copyright (c) 2018, ETH Zurich (Switzerland)
% All rights reserved.
% Published under the BSD-3-Clause License.

% This scrit has been developed within ETH Zurich, Department of Biosystems Science and Engineering by Leo Kunz
% Last update: 14.12.2018 (LK) 

function varargout = DDD(varargin)
% DDD MATLAB code for DDD.fig
%      DDD, by itself, creates a new DDD or raises the existing
%      singleton*.
%
%      H = DDD returns the handle to a new DDD or the handle to
%      the existing singleton*.
%
%      DDD('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in DDD.M with the given input arguments.
%
%      DDD('Property','Value',...) creates a new DDD or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before DDD_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to DDD_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE'_S Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help DDD

% Last Modified by GUIDE v2.5 17-Dec-2018 09:30:29

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @DDD_OpeningFcn, ...
                   'gui_OutputFcn',  @DDD_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before DDD is made visible.
function DDD_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to DDD (see VARARGIN)

% Choose default command line output for DDD
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes DDD wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = DDD_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in GO_Btn.
function GO_Btn_Callback(hObject, eventdata, handles)
% hObject    handle to GO_Btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
Sigma = str2double(get(handles.sigma_edittxt,'String'));
FilterSize_um = str2double(get(handles.Filtersize_edittxt,'String'));
pixelsize = str2double(get(handles.psize_edittxt,'String'));
zslice = (str2double(get(handles.z_size,'String')));
FilterSize = 2*floor((FilterSize_um/pixelsize)/2)-1;

%handles.zslices = str2double(get(handles.zsize_edittxt,'String'));

[filename, pathname] = ...
    uiputfile('*.tif', 'Name the heatmap. DDD will create a folder to store heatmap and quantification.');
addpath(pathname)

mkdir([pathname '\density_channel ' filename(1:end-4)])
newdir = [pathname 'density_channel ' filename(1:end-4)];

set(DDD, 'pointer', 'watch')
drawnow;

pre_zFilter = round(FilterSize/(1/(pixelsize)));
zFilter = 2*floor(pre_zFilter/2)-1;
Sigmaz = (Sigma/(zslice/pixelsize));

% imports the 3D Matrix where the centroids are true and the rest is false
img = handles.FinalImages3D;

% doing the assymetric gaussian 3D blur
volSmooth = imgaussfilt3(img, [Sigma Sigma Sigmaz],...
    'FilterSize', [FilterSize FilterSize zFilter],...
    'Padding',0,...
    'FilterDomain','frequency');
    
% Retrieving the maximum value of the heatmap as an adjustment factor 
Maxi = max(max(max((volSmooth(:,:,:)))));
toexport = Maxi;
toexport_c = num2cell(toexport);
heading = {'adjustment factor'};
tosave = [heading; toexport_c];
xlswrite([newdir '\' 'HM_adjustmentfactor.xlsx'],tosave,1)
clearvars toexport toexport_c heading tosave

% Normalize heatmap

if get(handles.radiobutton6,'value') == 1 
    % Normalize heatmap to highest value of other heatmao (adjustment
    % factor)
    % add one voxel that has a global max value to circumvent
    % stretching by vizualisation software
    adjfact = str2double(get(handles.adjfact,'String'));
    
    if adjfact < Maxi
        msgbox('You are trying to adjust the heatmap to a lower value. This will result in saturation of high denisty values in this heatmap.')
    end
    
    vSnorm = volSmooth./adjfact;
    aq = find(vSnorm < 0);
    vSnorm(aq) = 0;

    vSnorm_8bit = vSnorm*255;
    volSmooth_8bit = uint8(vSnorm_8bit);
    volSmooth_8bit(1,1,1) = 255;
    volSmooth_8bit(1,1,2) = 0;
else
    % Normalize heatmap to highest value of itself
    vSnorm = volSmooth./Maxi;
    aq = find(vSnorm < 0);
    vSnorm(aq) = 0;

    vSnorm_8bit = vSnorm*255;
    volSmooth_8bit = uint8(vSnorm_8bit);
end

% Write heatmap to file (tiff in location specified before)
imwrite(volSmooth_8bit(:,:,1),[newdir '\' filename])
for i = 2:handles.handles.zslices
    imwrite(volSmooth_8bit(:,:,i),[newdir '\' filename],'WriteMode','append')
end

%% Downstream analysis of heatmap

if get(handles.radiobutton1,'value') == 1  % if quantification is wanted on top of the heatmap, this is done here 
    
    XYZxyz = handles.XYZxyz;
    clearvars volSmooth vSnorm_8bit img aq

    % get volume of all densities
    levels = unique(volSmooth_8bit);
    howmanylevels = length(levels);
    counter = zeros(howmanylevels,1);
    counter_allabove = zeros(howmanylevels,1);
    volumeofarea_allabove = zeros(howmanylevels,1);
    concfromspots = zeros(howmanylevels,1);

    f = waitbar(0,'Please wait...');

    for i =1:howmanylevels

        howmanypix1 = length(find(volSmooth_8bit >= levels(i,1)));
        nn = howmanypix1 * ((str2double(get(handles.psize_edittxt,'String')))^2)*(str2double(get(handles.z_size,'String')));
        volumeofarea_allabove(i,1) = nn;

        
        pix = (volSmooth_8bit >= levels(i,1));
        
        % counts ddd/centroids within hotspots
        count = 0;
        for j = 1:size(XYZxyz,1)
            count = count + pix(XYZxyz(j,5),XYZxyz(j,4),XYZxyz(j,6));
        end
        counter(i,1) = count;

        CC1 = bwconncomp(pix,6);    
        counter_allabove(i,1) = CC1.NumObjects;

        waitbar(i/howmanylevels)   
    end
    
    close(f)
    
    % Calcualting the concentration in hotspots and all the non-hotspot
    % areas with the threshold being i, the value of 1 to 256 for 8bit
    %for i = 1:howmanylevels
    %    vol_nohotspot = (volumeofarea_allabove(1)) - (volumeofarea_allabove(i));
    %    spots_nohotspots = counter(1) - counter(i);
        
    %    concfromspots(i,1) = ((counter(i)./(volumeofarea_allabove(i)./1000000000000000))/(6.022141*(10^23)))*1000000000000;
    %    conc_no_spots(i,1) = ((spots_nohotspots./(vol_nohotspot./1000000000000000))/(6.022141*(10^23)))*1000000000000;
    %end
   

    
    if get(handles.radiobutton2,'value') == 1 %manual thresholding selected
        hotspot_threshold = (str2double(get(handles.mantresh,'String')));
        %ints = concfromspots < ceil(manual_treshold+2) & concfromspots > floor(manual_treshold-2);
        
        %if sum(ints) == 0
            msgbox('The concentration threshold value cannot be found in this heatmap. Try a lower threshold.')
        %else
        %    ints_ind = find(ints == 1);
        %    hotspot_threshold = ints_ind(1,1);
       % end
        
        
    elseif get(handles.radiobutton4,'value') == 1
        % normalize the hotspot count data
        ma = max(counter_allabove);
        allabove_norm = counter_allabove./ma;

        % take the derivate of the hotspot count data distribution at any point 
        der_allabove = gradient(allabove_norm,(1:256));

        % normalize the derivative data
        max_der_allabove_norm = max(der_allabove);
        der_allabove_norm = der_allabove./max_der_allabove_norm;

        % smooth derivative data
        der_allabove_norm_smooth = smooth(der_allabove_norm,20);

        % exluding edges, find the minimum of the derivative distribution i.e. the
        % point of the distribution where the decline of number of putative
        % hotspots is greatest - use this to define that all intensity values in
        % the 8 bit image above the trheshold are regarded hotspots
        mini = min(der_allabove_norm_smooth(3:253));
        hotspot_threshold = find(der_allabove_norm_smooth == mini);
        
        if length(hotspot_threshold) > 1
            hotspot_threshold = hotspot_threshold(1,1);
        end

        figgi = figure(); set(figgi, 'Visible', 'off');
        plot(der_allabove_norm_smooth,'LineWidth',2,'Color',[0,0,0])
        hold on
        plot(allabove_norm,'LineWidth',2,'Color','r')
        hold off
        axis square
        xlim([0 256])
        print(figgi,[newdir '\' 'Numberofhotspot_smoothed1stderivative.tif'],'-dtiff','-r300')
        close(figgi)

        toexport = [double(levels) allabove_norm der_allabove_norm_smooth];
        toexport_c = num2cell(toexport);
        heading = {'Levels (in 8 bit)' 'Normalized number of hotspots at given intensity' 'Smoothed first derviative of normalized number of hotspots'};
        tosave_NoHD = [heading; toexport_c];
        xlswrite([newdir '\' 'Number_of_hotspots_Derivative.xlsx'],tosave_NoHD,1)
        clearvars toexport toexport_c heading tosave_NoHD

    elseif get(handles.radiobutton3,'value') == 1
        % find the maximum value in the hotspot counter curve, the
        % point where hotspots start to fuse when the intensity
        % threshold level drops
        
        [~, locs] = findpeaks(counter_allabove,'MinPeakProminence',10);
        
        hotspot_threshold = locs(end);
        
        %ma = max(counter_allabove);
        %hotspot_threshold = find(counter_allabove == ma);

        if length(hotspot_threshold) > 1
            hotspot_threshold = hotspot_threshold(1,1);
        end
        
        figgi = figure(); set(figgi, 'Visible', 'off');
        plot(counter_allabove,'LineWidth',2,'Color',[0,0,0])
        axis square
        xlim([0 256])
        print(figgi,[newdir '\' 'Numberofhotspots.tif'],'-dtiff','-r300')

        close(figgi)
            
    elseif get(handles.radiobutton5,'value') == 1  
        % calcualting the concentration outside hotspots
        conc_no_spots2 =  conc_no_spots./max(conc_no_spots);

        der1 = (diff(conc_no_spots2,1));
        derN1 = der1./max(der1);
        sderN1 = smooth(derN1,5);

        % the second derivative of the curve of non-hotspot concentration shows
        % where the curve starts to get saturated i.e. the bacakgroudn
        % concntration does not increase exponentially anymore but start to
        % approximate its highest value
        der2 = (diff(sderN1,1));
        derN2 = der2./max(der2);
        sderN2 = smooth(derN2,5);

        hotspot_threshold = find(sderN2 == min(sderN2));
        
        if length(hotspot_threshold) > 1
            hotspot_threshold = hotspot_threshold(1,1);
        end

        counter_allabove2 = counter_allabove./max(counter_allabove);

        figgi = figure(); set(figgi, 'Visible', 'off');
        plot(counter_allabove2,'LineWidth',2,'Color',[0,0,0])
        hold on
        plot(conc_no_spots2,'LineWidth',2,'Color','r')
        hold on
        plot(sderN2,'LineWidth',2,'Color','b')
        axis square
        xlim([0 256])
        print(figgi,[newdir '\' 'Numberofhotspots.tif'],'-dtiff','-r300')

        close(figgi)
	end
        

    % concentration in pmol/l at threshold level, hot and cold spot
    %concfromspots1 = ((counter(hotspot_threshold)./(volumeofarea_allabove(hotspot_threshold)./1000000000000000))/(6.022141*(10^23)))*1000000000000;
    %vol_nohotspot = (volumeofarea_allabove(2)) - (volumeofarea_allabove(hotspot_threshold));
    %spots_nohotspots = counter(2) - counter(hotspot_threshold);
    %conc_no_spots1 = ((spots_nohotspots./(vol_nohotspot./1000000000000000))/(6.022141*(10^23)))*1000000000000;

    
    %toexport = [double(levels) volumeofarea_allabove];
    %toexport_c = num2cell(toexport);
    %heading = {'Levels (in 8 bit)' 'Volume of hotspots'};
    %tosave_VoH = [heading; toexport_c];
    %xlswrite([newdir '\' filename(1:end-4) '_Volume_of_hotspots.xlsx'],tosave_VoH,1)
    %clearvars toexport toexport_c heading tosave_VoH

    %toexport = [hotspot_threshold concfromspots1 conc_no_spots1];
    %toexport_c = num2cell(toexport);
    %heading = {'Hotspot treshold level (in 8 bit)' 'Average concentration within hotspots (pmol/l)' 'Average concentration outside hotspots (pmol/l)'};
    %tosave_CiH = [heading; toexport_c];
    %xlswrite([newdir '\' filename(1:end-4) '_Concentration_in_hotspots.xlsx'],tosave_CiH,1)
    %clearvars toexport toexport_c heading tosave_CiH

    
    
    % Translate the hotspot threshold, filter out smallest hotspots, save the distribution of volume of individual hotspots
    
    % define what the hotspots in 3D
    hotspots = volSmooth_8bit >= levels(hotspot_threshold+1,1);
    
    % calcualte the centroid of the hotspot
    CC = bwconncomp(hotspots,6); 
    stats_hs = regionprops3(CC,'Volume','Centroid');
    vol = cat(1, stats_hs.Volume);
    
    % hotspot size threshold, default is 100 um3 
    volthresh = str2double(get(handles.sizethresh,'String'));
    
    vol2 = vol.*((str2double(get(handles.psize_edittxt,'String')))^2)*(str2double(get(handles.z_size,'String')));
   
    % smallest hotspots are filtered out
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ind_above = find(vol2 > volthresh);
    
    vol_good = vol2(ind_above);
    
    toexport = [double(hotspot_threshold) length(vol_good)];
    toexport_c = num2cell(toexport);
    heading = {'Hotspot threshold' 'Number of hotspots'};
    tosave_NoH = [heading; toexport_c];
    xlswrite([newdir '\' 'Number_of_hotspots.xlsx'],tosave_NoH,1)
    
    a = find(vol_good < 1000);
    b = find(vol_good > 1000 & vol_good < 10000);   
    c = find(vol_good > 10000);   
    
    vol_good_below1000 = vol_good(a);
    vol_good_betw1000_10000 = vol_good(b);   
    vol_good_above10000 = vol_good(c);
   
    toexport_vg = num2cell(vol_good);
    toexport_vg2 = num2cell(vol_good_below1000);
    toexport_vg3 = num2cell(vol_good_betw1000_10000);
    toexport_vg4 = num2cell(vol_good_above10000);
    headingvg = {'Volume of hotspots in um3'};
    headingvg2 = {'Volume of hotspots in um3; below 1000 um3'};
    headingvg3 = {'Volume of hotspots in um3; above 1000, belwow 10000 um3'};
    headingvg4 = {'Volume of hotspots in um3; above 100000 um3'};
    tosave_vg = [headingvg; toexport_vg];
    tosave_vg4 = [headingvg2; toexport_vg4];
    tosave_vg3 = [headingvg3; toexport_vg3];
    tosave_vg2 = [headingvg4; toexport_vg2];
    
    xlswrite([newdir '\' 'Volume_of_individual_hotspots.xlsx'],tosave_vg4,4)
    xlswrite([newdir '\' 'Volume_of_individual_hotspots.xlsx'],tosave_vg3,3)
    xlswrite([newdir '\' 'Volume_of_individual_hotspots.xlsx'],tosave_vg2,2)
    xlswrite([newdir '\' 'Volume_of_individual_hotspots.xlsx'],tosave_vg,1)
    
end
    
%% Directionality analysis if wanted

if get(handles.radiobutton8,'value') == 1  % if directionality analysis is wanted on top of the heatmap, this is done here 
    
    answer1 = questdlg('Do you have a distance map to compare the heatmap to?', ...
	'Distance map confirmation', 'Yes','No','No');
    
    if strcmp(answer1,'No')
        
        f = msgbox('To use this function please create a distance map first.');
        
    else
    
        % load 3D distance map
        [filename, pathname] = ...
            uigetfile({'*.tif';'*.*'}, 'Select distance map tiff files', 'MultiSelect', 'on');
        addpath(pathname)
        ch_num = 1;
        filenameImg_allCh_DT = cell(1,ch_num);

        % filenames for every channel in a separate array
        for i = 1:length(filename)
            if i <= str2double(get(handles.zsize_edittxt,'String'))
                for j = 1:str2double(get(handles.zsize_edittxt,'String'))
                    filenameImg_Ch0{1,j} = filename{1,j};
                end
                filenameImg_allCh_DT{1,1} = filenameImg_Ch0;
            elseif i > str2double(get(handles.zsize_edittxt,'String')) & i <= (str2double(get(handles.zsize_edittxt,'String'))*2)
                for j = (str2double(get(handles.zsize_edittxt,'String'))+1):(str2double(get(handles.zsize_edittxt,'String'))*2)
                    filenameImg_Ch1{1,j-str2double(get(handles.zsize_edittxt,'String'))} = filename{1,j};
                end
                filenameImg_allCh_DT{1,2} = filenameImg_Ch1;
            end
        end

        % now I have the filenames for every channel in a separate array, now I can
        % read the tiffs in separatly
        FileTif_DT = filenameImg_allCh_DT{1,1}{1,1};
        InfoImage=imfinfo(FileTif_DT);
        mImage=InfoImage(1).Width;
        nImage=InfoImage(1).Height;
        for i = 1:ch_num
            FinalImages3D_DT{i,1} = zeros(nImage,mImage,str2double(get(handles.zsize_edittxt,'String')),'uint8'); 
        end
        clearvars FileTif InfoImage mImage nImage

        FinalImage_DT = cell(handles.zslices,1);
        FinalImages_DT = cell(ch_num,1);

        for i = 1:ch_num
            for j = 1:handles.zslices
                FileTif_DT = filenameImg_allCh_DT{1,i}{1,j};
                FinalImage_DT{j,1} = loadtiff(FileTif_DT);  
            end
            FinalImages_DT{i,1} = FinalImage_DT;
            for ii = 1:handles.zslices
                FinalImages3D_DT{i,1}(:,:,ii) = FinalImages_DT{i,1}{ii,1};
            end
        end
        clearvars FinalImage FinalImages

        
        answer2 = questdlg('Do you want to load a tissue mask?', ...
        'Tissue mask', 'Yes','No','No');

        if strcmp(answer2,'Yes')
            % take 3D mask of PLA volume
            [filename, pathname] = ...
                uigetfile({'*.tif';'*.*'}, 'Select tissue mask tiff files', 'MultiSelect', 'on');
            addpath(pathname)
            ch_num = 1;
            filenameImg_allCh = cell(1,ch_num);

            % filenames for every channel in a separate array
            for i = 1:length(filename)
                if i <= handles.zslices
                    for j = 1:handles.zslices
                        filenameImg_Ch0{1,j} = filename{1,j};
                    end
                    filenameImg_allCh{1,1} = filenameImg_Ch0;
                elseif i > handles.zslices & i <= (handles.zslices*2)
                    for j = (handles.zslices+1):(handles.zslices*2)
                        filenameImg_Ch1{1,j-handles.zslices} = filename{1,j};
                    end
                    filenameImg_allCh{1,2} = filenameImg_Ch1;
                end
            end

            % now I have the filenames for every channel in a separate array, now I can
            % read the tiffs in separatly
            FileTif = filenameImg_allCh{1,1}{1,1};
            InfoImage=imfinfo(FileTif);
            mImage=InfoImage(1).Width;
            nImage=InfoImage(1).Height;
            for i = 1:ch_num
                FinalImages3D{i,1} = zeros(nImage,mImage,handles.zslices,'uint8'); 
            end
            clearvars FileTif InfoImage mImage nImage

            FinalImage = cell(handles.zslices,1);
            FinalImages = cell(ch_num,1);

            for i = 1:ch_num
                for j = 1:handles.zslices
                    FileTif = filenameImg_allCh{1,i}{1,j};
                    FinalImage{j,1} = loadtiff(FileTif);  
                end
                FinalImages{i,1} = FinalImage;
                for ii = 1:handles.zslices
                    FinalImages3D{i,1}(:,:,ii) = FinalImages{i,1}{ii,1};
                end
            end
            clearvars FinalImage FinalImages
        else
            FakeMask = ones(size(volSmooth_8bit,1),size(volSmooth_8bit,2),size(volSmooth_8bit,3));
            FinalImages3D{1,1} = FakeMask.*255;
        end
        
        XYZxyz = handles.XYZxyz;
        
        Xsize = size(handles.FinalImages3D,1);
        Ysize = size(handles.FinalImages3D,2);
        Zsize = size(handles.FinalImages3D,3);
        
        matrix = {zeros(Ysize,Xsize,Zsize)};
        FinalPosMat3D = matrix{1,1};
        % make matrix out of coordinates
        for i = 1:size(XYZxyz,1)
            j = XYZxyz(i,5);
            k = XYZxyz(i,4);
            l = XYZxyz(i,6);

            FinalPosMat3D(j,k,l) = FinalPosMat3D(j,k,l)+1;
        end   
        
        voxelsize_xy = str2double(get(handles.psize_edittxt,'String'));
        voxelsize_z = str2double(get(handles.z_size,'String'));
        
        % take "cells" (spheres) of different diameters (5, 10, 15, 20)
        radius_in_vox_xy = ceil(str2double(get(handles.radius_txt,'String'))/voxelsize_xy)-1; 
        radius_in_vox_z = ceil(str2double(get(handles.radius_txt,'String'))/voxelsize_z)-1; 
        
        a = size(FinalPosMat3D,1);
        b = size(FinalPosMat3D,2);
        c = size(FinalPosMat3D,3);

        FinalPosMat3D_new = FinalPosMat3D;

        FinalPosMat3D_new(1:20,:,:) = 0;
        FinalPosMat3D_new(end-20:end,:,:) = 0;
        FinalPosMat3D_new(:,1:20,:) = 0;
        FinalPosMat3D_new(:,end-20:end,:) = 0;
        FinalPosMat3D_new(:,:,1) = 0;
        FinalPosMat3D_new(:,:,end) = 0;
        
        Maxi_DT = 250;
        vSnorm_DT = FinalImages3D_DT{1,1};
        aq = find(vSnorm_DT <= 0);
        vSnorm_DT(aq) = 255;
        aq = find(vSnorm_DT > Maxi_DT);
        vSnorm_DT(aq) = 255;

        DT_8bit = uint8(vSnorm_DT);
        
        sep = str2double(get(handles.spacing_txt,'String')); 
        sep_z = str2double(get(handles.spacingZ_txt,'String')); 
        
        % place the probe inside the mask volume (ordered placement)
        % place/shift the center point by "radius" voxels 

        TwoDgrid = zeros(size(FinalPosMat3D,1), size(FinalPosMat3D,2));
        for i = sep:sep:size(TwoDgrid,1)-sep
            for j = sep:sep:size(TwoDgrid,2)-sep
                TwoDgrid(i,j) = 1;
            end
        end

        IndOnes = find(TwoDgrid == 1);

        counter = 0;
        clearvars array
        
        for i = 1:sep_z:size(FinalPosMat3D,3)

            TwoDquery = FinalImages3D{1,1}(:,:,i);
            TwoDquery = double(TwoDquery);

            ToComp = find(TwoDquery==255);

            [C] = intersect(IndOnes,ToComp);

            [Coord1, Coord2] = ind2sub([size(TwoDquery)], C);

    
            for j = 1:length(Coord1)    
                Coord2min = floor(Coord2(j)-radius_in_vox_xy);
                Coord2max = ceil(Coord2(j)+radius_in_vox_xy);
                Coord1min = floor(Coord1(j)-radius_in_vox_xy);
                Coord1max = ceil(Coord1(j)+radius_in_vox_xy);
                zmin = floor(i-radius_in_vox_z);
                zmax = ceil(i+radius_in_vox_z);

                if Coord2min <= 0
                    Coord2min = 1;
                end
                if Coord2max > size(volSmooth_8bit,2)
                    Coord2max = size(volSmooth_8bit,2);
                end
                if Coord1min <= 0
                    Coord1min = 1;
                end
                if Coord1max > size(volSmooth_8bit,1)
                    Coord1max = size(volSmooth_8bit,1);
                end
                if zmin <= 0
                    zmin = 1;
                end
                if zmax > size(FinalPosMat3D,3)
                    zmax = size(FinalPosMat3D,3);
                end
        
                probeVolume_Dens = vSnorm(Coord1min:Coord1max,Coord2min:Coord2max,zmin:zmax);
                center = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i-zmin];

                [columnsInImage rowsInImage pagesInImage] = meshgrid(1:size(probeVolume_Dens,1), 1:size(probeVolume_Dens,2), 1:size(probeVolume_Dens,3));
                % Next create the circle in the image.
                centerX = center(1);
                centerY = center(2);
                centerZ = center(3);
                sphereVoxels = (rowsInImage - centerY).^2./radius_in_vox_xy.^2 ...
                    + (columnsInImage - centerX).^2./radius_in_vox_xy.^2 ...
                    + (pagesInImage - centerZ).^2./radius_in_vox_z.^2 <= 1;

                probeVolume_Dens(~sphereVoxels) = NaN;

                Maxi2 = max(max(max((probeVolume_Dens(:,:,:)))));
                probeVolume_Dens2 = probeVolume_Dens./Maxi2;
                AvgInt = mean(probeVolume_Dens2(sphereVoxels));

                HiInt = probeVolume_Dens2 > 0.90;
                L = bwlabeln(HiInt);
                stats = regionprops3(L);
                
                centroids = cat(1, stats.Centroid);
                volume = cat(1, stats.Volume);

                arrayStart = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i];
                for k = 1:size(centroids,1)

                    a = volume(k,1);

                    if a < 2
                        volume2(k,1) = NaN;
                        distances2(k,1) = NaN;
                        arrayEnd = [arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                        X{k,1} = [arrayStart(1,1), arrayStart(1,2), arrayStart(1,3);arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                    else
                        arrayEnd = [centroids(k,1),centroids(k,2),centroids(k,3)];
                        X{k,1} = [arrayEnd(1,1), arrayEnd(1,2), arrayEnd(1,3); arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                        distances2(k,1) = pdist(X{k,1},'euclidean');
                        volume2(k,1) = volume(k,1);
                    end
                end

                if size(centroids,1) > 1 
                    if sum(isnan(distances2)) == size(centroids,1)
                        indm = 1;
                    else
                        mind = (max(distances2));
                        indm = distances2 == mind;
                        if sum(indm) > 1
                           indm = indm(1,1); 
                        end
                    end
                else
                    indm = 1;
                end
                
                counter = counter+1;
                array(counter,:) = [Coord2(j), Coord1(j), i ,centroids(indm,1)+Coord2min, centroids(indm,2)+Coord1min, centroids(indm,3)+zmin];

                if sum(array(counter,:)) == 0
                    array(counter,:) = [];
                    counter = counter-1;
                end


                clearvars MaxCoord2 MaxCoord1 MaxCoordZ Max dmin dminind probeVolume_Dens probeVolume dmax...
                    Coord2min Coord2max Coord1min Coord1max zmin zmax Pix TwoDquery newPix HiInt centroids...
                    L MScoreInd score centroids volume distance volume2 distances2 q probeVolume_Dens2 X indm

            end 

            clearvars Coord2 Coord1 Coords
        end

        clearvars arrayStart arrayEnd ArrayD
        arrayStart = [array(:,1), array(:,2), array(:,3)];
        arrayEnd =  [array(:,4), array(:,5), array(:,6)];
        ArrayD = arrayEnd-arrayStart;


        fig1 = figure();
        set(fig1,'Visible','off')
        q = quiver3(array(:,1),array(:,2),array(:,3), ArrayD(:,1), ArrayD(:,2), ArrayD(:,3),...
            1,'AlignVertexCenters','on','Color','c','LineWidth',2) ;
        r = atan2(q.VData,q.UData);
        degrees_HM = r*180/pi;
        
        close(fig1)

        % Calcualtions on distance map
        
        % place the probe inside the mask volume (ordered placement)
        % place/shift the center point by "radius" voxels 

        TwoDgrid_DT = zeros(size(FinalPosMat3D,1), size(FinalPosMat3D,2));
        for i = sep:sep:size(TwoDgrid_DT,1)-sep
            for j = sep:sep:size(TwoDgrid_DT,2)-sep
                TwoDgrid_DT(i,j) = 1;
            end
        end

        IndOnes_DT = find(TwoDgrid_DT == 1);

        counter_DT = 0;
        clearvars array_DT4

        % calculate Vector map on distance map

        for i = 1:sep_z:size(FinalPosMat3D,3)

            TwoDquery_DT = FinalImages3D{1,1}(:,:,i);
            TwoDquery_DT = double(TwoDquery_DT);

            ToComp = find(TwoDquery_DT==255);

            [C] = intersect(IndOnes_DT,ToComp);

            [Coord1, Coord2] = ind2sub([size(TwoDquery_DT)], C);

            for j = 1:length(Coord1)    
                Coord2min = floor(Coord2(j)-radius_in_vox_xy);
                Coord2max = ceil(Coord2(j)+radius_in_vox_xy);
                Coord1min = floor(Coord1(j)-radius_in_vox_xy);
                Coord1max = ceil(Coord1(j)+radius_in_vox_xy);
                zmin = floor(i-radius_in_vox_z);
                zmax = ceil(i+radius_in_vox_z);

                if Coord2min <= 0
                    Coord2min = 1;
                end
                if Coord2max > size(DT_8bit,2)
                    Coord2max = size(DT_8bit,2);
                end
                if Coord1min <= 0
                    Coord1min = 1;
                end
                if Coord1max > size(DT_8bit,1)
                    Coord1max = size(DT_8bit,1);
                end
                if zmin <= 0
                    zmin = 1;
                end
                if zmax > size(DT_8bit,3)
                    zmax = size(DT_8bit,3);
                end

                probeVolume_Dens_DT = DT_8bit(Coord1min:Coord1max,Coord2min:Coord2max,zmin:zmax);
                center = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i-zmin];

                [columnsInImage rowsInImage pagesInImage] = meshgrid(1:size(probeVolume_Dens_DT,1), 1:size(probeVolume_Dens_DT,2), 1:size(probeVolume_Dens_DT,3));
                % Next create the circle in the image.
                centerX = center(1);
                centerY = center(2);
                centerZ = center(3);
                sphereVoxels = (rowsInImage - centerY).^2./radius_in_vox_xy.^2 ...
                    + (columnsInImage - centerX).^2./radius_in_vox_xy.^2 ...
                    + (pagesInImage - centerZ).^2./radius_in_vox_z.^2 <= 1;

    
                probeVolume_Dens_DT(~sphereVoxels) = 0;

                Maxi2 = max(max(max((probeVolume_Dens_DT(:,:,:)))));
                probeVolume_Dens_DT(~sphereVoxels) = 255;
                probeVolume_Dens2_DT = double(probeVolume_Dens_DT)./double(Maxi2);

                probeVolume_Dens3_DT = 1./probeVolume_Dens2_DT;
                Maxi2 = max(max(max((probeVolume_Dens3_DT(:,:,:)))));
                probeVolume_Dens2_DT = double(probeVolume_Dens3_DT)./double(Maxi2);

                AvgInt = mean(probeVolume_Dens2_DT(sphereVoxels));
                LowInt = probeVolume_Dens2_DT > 0.95;
                L = bwlabeln(LowInt);
                stats = regionprops3(L);
                centroids = cat(1, stats.Centroid);
                volume = cat(1, stats.Volume);


                arrayStart_DT = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i];
                for k = 1:size(centroids,1)

                    a = volume(k,1);

                    if a < 2
                        volume2(k,1) = 0;
                        distances2(k,1) = 1;

                    else
                        arrayEnd_DT = [centroids(k,1),centroids(k,2),centroids(k,3)];
                        X = [arrayEnd_DT(1,1), arrayEnd_DT(1,2), arrayEnd_DT(1,3); arrayStart_DT(1,1), arrayStart_DT(1,2), arrayStart_DT(1,3)];
                        distances2(k,1) = pdist(X,'euclidean');
                        volume2(k,1) = volume(k,1);
                    end
                end

                if size(centroids,1) > 1 
                    if sum(isnan(distances2)) == size(centroids,1)
                        indm = 1;
                    else
                        mind = (max(distances2));
                        indm = distances2 == mind;
                        if sum(indm) > 1
                           indm = indm(1,1); 
                        end
                    end
                else
                    indm = 1;
                end
                
                counter_DT = counter_DT+1;
                
                array_DT4(counter_DT,:) = [Coord2(j), Coord1(j), i ,centroids(indm,1)+Coord2min, centroids(indm,2)+Coord1min, centroids(indm,3)+zmin];

                clearvars MaxCoord2 MaxCoord1 MaxCoordZ Max dmin dminind probeVolume_Dens_DT probeVolume_DT dmax...
                    Coord2min Coord2max Coord1min Coord1max zmin zmax Pix TwoDquery_DT newPix LowInt centroids...
                    L MScoreInd_DT score centroids volume distance volume2 distances2 q probeVolume_Dens2_DT probeVolume_Dens3_DT
            end 

            clearvars Coord2 Coord1 Coords
        end
        
        clearvars arrayStart_DT4 arrayEnd_DT4 ArrayD_DT4
        arrayStart_DT = [array_DT4(:,1), array_DT4(:,2), array_DT4(:,3)];
        arrayEnd_DT =  [array_DT4(:,4), array_DT4(:,5), array_DT4(:,6)];
        ArrayD_DT4 = arrayEnd_DT-arrayStart_DT;

        fig2 = figure();
        set(fig2,'Visible','off')
        q_DT = quiver3(array_DT4(:,1),array_DT4(:,2),array_DT4(:,3), ArrayD_DT4(:,1), ArrayD_DT4(:,2), ArrayD_DT4(:,3),...
            1,'AlignVertexCenters','on','Color','c','LineWidth',1.5); 

        clearvars degrees_DT 
        r_DT = atan2(q_DT.VData,q_DT.UData);
        degrees_DT = r_DT*180/pi;

        close(fig2)
        
        % calculate difference in angles
        clearvars diff_in_angle codi codi_dist co  
        diff_in_angle = min((360) - abs(degrees_HM - degrees_DT), abs(degrees_HM - degrees_DT));
        codi = cosd(diff_in_angle);
        
        codi_dist = [codi array_DT4(:,6)];

        U = unique(array_DT4(:,6));

        for i = 1:255
            if sum(i == U) > 0
                co = codi_dist(:,2) == i; 
                avg(i,1) = mean(codi_dist(co,1));
            end
        end

        
        % save matrix of distance resolved alignments of heatmap and
        % distance map
        clearvars toexport toexport_c heading tosave
        toexport = [codi_dist];
        toexport_c = num2cell(toexport);
        heading = {'Cosine of the angle between distances map and heatmap' 'Distance to strucutre at the starting point'};
        tosave = [heading; toexport_c];
        xlswrite([newdir '\' 'VectorAlignment.xlsx'],tosave,1)
        
        clearvars toexport toexport_c heading tosave
        toexport = [(1:1:length(avg))' avg];
        toexport_c = num2cell(toexport);
        heading = {'Distance to X', 'Average cosine of the angle between distances map and heatmap'};
        tosave = [heading; toexport_c];
        xlswrite([newdir '\' 'AverageVectorAlignment.xlsx'],tosave,1)
    end
end
%% Migration analysis if wanted

if get(handles.radiobutton9,'value') == 1  % if migration analysis is wanted on top of the heatmap, this is done here 
    
    answer2 = questdlg('Do you want to load a tissue mask?', ...
        'Tissue mask', 'Yes','No','No');
    
     if strcmp(answer2,'Yes')
            % take 3D mask of PLA volume
            [filename, pathname] = ...
                uigetfile({'*.tif';'*.*'}, 'Select tissue mask tiff files', 'MultiSelect', 'on');
            addpath(pathname)
            ch_num = 1;
            filenameImg_allCh = cell(1,ch_num);

            % filenames for every channel in a separate array
            for i = 1:length(filename)
                if i <= handles.zslices
                    for j = 1:handles.zslices
                        filenameImg_Ch0{1,j} = filename{1,j};
                    end
                    filenameImg_allCh{1,1} = filenameImg_Ch0;
                elseif i > handles.zslices & i <= (handles.zslices*2)
                    for j = (handles.zslices+1):(handles.zslices*2)
                        filenameImg_Ch1{1,j-handles.zslices} = filename{1,j};
                    end
                    filenameImg_allCh{1,2} = filenameImg_Ch1;
                end
            end

            % now I have the filenames for every channel in a separate array, now I can
            % read the tiffs in separatly
            FileTif = filenameImg_allCh{1,1}{1,1};
            InfoImage=imfinfo(FileTif);
            mImage=InfoImage(1).Width;
            nImage=InfoImage(1).Height;
            for i = 1:ch_num
                FinalImages3D{i,1} = zeros(nImage,mImage,handles.zslices,'uint8'); 
            end
            clearvars FileTif InfoImage mImage nImage

            FinalImage = cell(handles.zslices,1);
            FinalImages = cell(ch_num,1);

            for i = 1:ch_num
                for j = 1:handles.zslices
                    FileTif = filenameImg_allCh{1,i}{1,j};
                    FinalImage{j,1} = loadtiff(FileTif);  
                end
                FinalImages{i,1} = FinalImage;
                for ii = 1:handles.zslices
                    FinalImages3D{i,1}(:,:,ii) = FinalImages{i,1}{ii,1};
                end
            end
            clearvars FinalImage FinalImages
        else
            FakeMask = ones(size(volSmooth_8bit,1),size(volSmooth_8bit,2),size(volSmooth_8bit,3));
            FinalImages3D{1,1} = FakeMask.*255;
        end
        
        XYZxyz = handles.XYZxyz;
        
        Xsize = size(handles.FinalImages3D,1);
        Ysize = size(handles.FinalImages3D,2);
        Zsize = size(handles.FinalImages3D,3);
        
        matrix = {zeros(Ysize,Xsize,Zsize)};
        FinalPosMat3D = matrix{1,1};
        % make matrix out of coordinates
        for i = 1:size(XYZxyz,1)
            j = XYZxyz(i,5);
            k = XYZxyz(i,4);
            l = XYZxyz(i,6);

            FinalPosMat3D(j,k,l) = FinalPosMat3D(j,k,l)+1;
        end   
        
        voxelsize_xy = str2double(get(handles.psize_edittxt,'String'));
        voxelsize_z = str2double(get(handles.z_size,'String'));
        % take "cells" (spheres) of different diameters (5, 10, 15, 20)
        radius_in_vox_xy = ceil(str2double(get(handles.radius_txt,'String'))/voxelsize_xy)-1; 
        radius_in_vox_z = ceil(str2double(get(handles.radius_txt,'String'))/voxelsize_z)-1; 
        
        a = size(FinalPosMat3D,1);
        b = size(FinalPosMat3D,2);
        c = size(FinalPosMat3D,3);

        FinalPosMat3D_new = FinalPosMat3D;

        FinalPosMat3D_new(1:20,:,:) = 0;
        FinalPosMat3D_new(end-20:end,:,:) = 0;
        FinalPosMat3D_new(:,1:20,:) = 0;
        FinalPosMat3D_new(:,end-20:end,:) = 0;
        FinalPosMat3D_new(:,:,1) = 0;
        FinalPosMat3D_new(:,:,end) = 0;
        
        sep = str2double(get(handles.spacing_txt,'String')); 
        sep_z = str2double(get(handles.spacingZ_txt,'String')); 
        
        % place the probe inside the mask volume (ordered placement)
        % place/shift the center point by "radius" voxels 

        TwoDgrid = zeros(size(FinalPosMat3D,1), size(FinalPosMat3D,2));
        for i = sep:sep:size(TwoDgrid,1)-sep
            for j = sep:sep:size(TwoDgrid,2)-sep
                TwoDgrid(i,j) = 1;
            end
        end

        IndOnes = find(TwoDgrid == 1);
        Dists_um = cell(1,length(1:3:size(FinalPosMat3D,3)));

        for i = 1:sep_z:size(FinalPosMat3D,3)

            TwoDquery = FinalImages3D{1,1}(:,:,i);
            TwoDquery = double(TwoDquery);

            ToComp = find(TwoDquery==255);

            [C] = intersect(IndOnes,ToComp);

            [Coord1, Coord2] = ind2sub([size(TwoDquery)], C);


            for j = 1:length(Coord1) 

                clearvars stats L Coord2min Coord2max Coord1min Coord2max zmin zmax...
                    probeVolume_Dens center columnsInImage rowsInImage pagesInImage...
                    sphereVoxels centerX centerY centerZ probeVolume_Dens2 Maxi2 HiInt...
                    centroids volume mind indm distances2 distances3 distances4...
                    volume2 volume3 volume4 X X2 X3 X4 dir dir2 dir3 dir4 newcenter...
                    newcenter2 Displacement 

                Coord2min = floor(Coord2(j)-radius_in_vox_xy);
                Coord2max = ceil(Coord2(j)+radius_in_vox_xy);
                Coord1min = floor(Coord1(j)-radius_in_vox_xy);
                Coord1max = ceil(Coord1(j)+radius_in_vox_xy);
                zmin = floor(i-radius_in_vox_z);
                zmax = ceil(i+radius_in_vox_z);

                if Coord2min <= 0
                    Coord2min = 1;
                end
                if Coord2max > size(volSmooth_8bit,2)
                    Coord2max = size(volSmooth_8bit,2);
                end
                if Coord1min <= 0
                    Coord1min = 1;
                end
                if Coord1max > size(volSmooth_8bit,1)
                    Coord1max = size(volSmooth_8bit,1);
                end
                if zmin <= 0
                    zmin = 1;
                end
                if zmax > size(FinalPosMat3D,3)
                    zmax = size(FinalPosMat3D,3);
                end

                probeVolume_Dens = vSnorm(Coord1min:Coord1max,Coord2min:Coord2max,zmin:zmax);
                center = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i-zmin];
                %center = size(probeVolume_Dens)/2+.5;
                
                %figure()
                %imshow(probeVolume_Dens(:,:,i))
                %colormap(jet(256))

                [columnsInImage rowsInImage pagesInImage] = meshgrid(1:size(probeVolume_Dens,1), 1:size(probeVolume_Dens,2), 1:size(probeVolume_Dens,3));
                % Next create the circle in the image.
                centerX = center(1);
                centerY = center(2);
                centerZ = center(3);
                sphereVoxels = (rowsInImage - centerY).^2./radius_in_vox_xy.^2 ...
                    + (columnsInImage - centerX).^2./radius_in_vox_xy.^2 ...
                    + (pagesInImage - centerZ).^2./radius_in_vox_z.^2 <= 1;

                probeVolume_Dens(~sphereVoxels) = NaN;

                Maxi2 = max(max(max((probeVolume_Dens(:,:,:)))));
                probeVolume_Dens2 = probeVolume_Dens./Maxi2;
                AvgInt = mean(probeVolume_Dens2(sphereVoxels));

                HiInt = probeVolume_Dens2 > 0.98;
                L = bwlabeln(HiInt);
                stats = regionprops3(L);

                centroids = cat(1, stats.Centroid);
                volume = cat(1, stats.Volume);


                arrayStart = [Coord2(j)-Coord2min, Coord1(j)-Coord1min,i];
                for k = 1:size(centroids,1)

                    a = volume(k,1);

                    if a < 2
                        volume2(k,1) = NaN;
                        distances2(k,1) = NaN;
                        arrayEnd = [arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                        X{k,1} = [arrayStart(1,1), arrayStart(1,2), arrayStart(1,3);arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                    else
                        arrayEnd = [centroids(k,1),centroids(k,2),centroids(k,3)];
                        X{k,1} = [arrayEnd(1,1), arrayEnd(1,2), arrayEnd(1,3); arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                        distances2(k,1) = pdist(X{k,1},'euclidean');
                        volume2(k,1) = volume(k,1);
                    end
                end

                if size(centroids,1) > 1 
                    
                    if sum(isnan(distances2)) == size(centroids,1)
                        distances2 = 0;
                        indm = 1;
                        clearvars X
                        X = [arrayStart(1,1), arrayStart(1,2), arrayStart(1,3);arrayStart(1,1), arrayStart(1,2), arrayStart(1,3)];
                    else
                        mind = (max(distances2));
                        indm = distances2 == mind;
                        if sum(indm) > 1
                           indm = indm(1,1); 
                        end
                        distances2 = distances2(indm);
                        volume2 = volume2(indm);
                        X = X{indm};
                    end
                    
                elseif isempty(centroids)
                    centroids = arrayStart;
                    Dist_um = 0;
                    Vecs = [0 0 0];
                    indm = 1;
                    X = [arrayStart; arrayStart];
                    distances2 = NaN;
                else
                    indm = 1;
                    X = X{indm};
                end

                dir2 = [centroids(indm,1)+Coord2min, centroids(indm,2)+Coord1min, centroids(indm,3)+zmin] - [center(1,1)+Coord2min, center(1,2)+Coord1min, center(1,3)+zmin];

                First_arrow = X(1,:)-X(2,:);
                Displacement = round(First_arrow);
                newcenter = [Coord2(j), Coord1(j),i] + Displacement; 

                %
                % now I know the distance of centroid to highest concentration and
                % also the direction 

                % distance traveled = distance2
                % direction traveled = X (end; start)
                % that means I can do this step again

                Xstart = [Coord2(j), Coord1(j),i];

                clearvars stats L Coord2min Coord2max Coord1min Coord2max zmin zmax...
                    probeVolume_Dens center columnsInImage rowsInImage pagesInImage...
                    sphereVoxels centerX centerY centerZ center probeVolume_Dens2 Maxi2 HiInt...
                    centroids volume mind indm columnsInImage rowsInImage pagesInImage 

                new_centroid = newcenter;
                new_i = (new_centroid(3));

                Coord2min = floor(new_centroid(1,1)-radius_in_vox_xy);
                Coord2max = ceil(new_centroid(1,1)+radius_in_vox_xy);
                Coord1min = floor(new_centroid(1,2)-radius_in_vox_xy);
                Coord1max = ceil(new_centroid(1,2)+radius_in_vox_xy);
                zmin = floor(new_i-radius_in_vox_z);
                zmax = ceil(new_i+radius_in_vox_z);

                if Coord2min <= 0
                    Coord2min = 1;
                end
                if Coord2max > size(volSmooth_8bit,2)
                    Coord2max = size(volSmooth_8bit,2);
                end
                if Coord1min <= 0
                    Coord1min = 1;
                end
                if Coord1max > size(volSmooth_8bit,1)
                    Coord1max = size(volSmooth_8bit,1);
                end
                if zmin <= 0
                    zmin = 1;
                end
                if zmax > size(FinalPosMat3D,3)
                    zmax = size(FinalPosMat3D,3);
                end

                probeVolume_Dens = vSnorm(Coord1min:Coord1max,Coord2min:Coord2max,zmin:zmax);
                center = [newcenter(1)-Coord2min, newcenter(2)-Coord1min,new_i-zmin];


                [columnsInImage, rowsInImage, pagesInImage] = meshgrid(1:size(probeVolume_Dens,1), 1:size(probeVolume_Dens,2), 1:size(probeVolume_Dens,3));
                % Next create the circle in the image.
                centerX = center(1);
                centerY = center(2);
                centerZ = center(3);
                sphereVoxels = (rowsInImage - centerY).^2./radius_in_vox_xy.^2 ...
                    + (columnsInImage - centerX).^2./radius_in_vox_xy.^2 ...
                    + (pagesInImage - centerZ).^2./radius_in_vox_z.^2 <= 1;

                probeVolume_Dens(~sphereVoxels) = NaN;


                Maxi2 = max(max(max((probeVolume_Dens(:,:,:)))));
                probeVolume_Dens2 = probeVolume_Dens./Maxi2;
                HiInt = probeVolume_Dens2 > 0.98;
                L = bwlabeln(HiInt);
                stats = regionprops3(L);

                centroids = cat(1, stats.Centroid);
                volume = cat(1, stats.Volume);

                clearvars arrayEnd

                arrayStart2 = center;
                for k = 1:size(centroids,1)
                    a = volume(k,1);
                    if a < 2
                        volume3(k,1) = NaN;
                        distances3(k,1) = NaN;  
                        arrayEnd = [arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3)];
                        X2{k,1} =  [arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3); arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3)];
                    else
                        arrayEnd = [centroids(k,1),centroids(k,2),centroids(k,3)];
                        X2{k,1} = [arrayEnd(1,1), arrayEnd(1,2), arrayEnd(1,3); arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3)];
                        distances3(k,1) = pdist(X2{k,1},'euclidean');
                        volume3(k,1) = volume(k,1);
                    end
                end

                if size(centroids,1) > 1 
                    if sum(isnan(distances3)) == size(centroids,1)
                        distances3 = NaN;
                        indm = 1;
                        X2 =  [arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3); arrayStart2(1,1), arrayStart2(1,2), arrayStart2(1,3)];
                    else
                        mind = (max(distances3));
                        indm = distances3 == mind;
                        if sum(indm) > 1
                           indm = indm(1,1); 
                        end
                        distances3 = distances3(indm);
                        volume3 = volume3(indm);
                        X2 = X2{indm};
                    end
                elseif isempty(centroids)
                    centroids = arrayStart2;
                    Dist_um = 0;
                    Vecs = [0 0 0];
                    indm = 1;
                    X2 = [arrayStart2; arrayStart2];
                    distances3 = NaN;
                else
                    indm = 1;
                    X2 = X2{indm};
                end

                dir3 = [centroids(indm,1)+Coord2min, centroids(indm,2)+Coord1min, centroids(indm,3)+zmin] - [new_centroid(1,1)+Coord2min, new_centroid(1,2)+Coord1min, new_centroid(1,3)+zmin];

                First_arrow = X(1,:)-X(2,:);
                Second_arrow = X2(1,:)-X2(2,:);

                Xq = [X(2,:);X2(2,:)];
                Xq2 = [First_arrow;Second_arrow];     

                figgi = figure();
                set(figgi, 'Visible', 'off')
                q = quiver3(Xq(:,1),Xq(:,2),Xq(:,3), Xq2(:,1), Xq2(:,2), Xq2(:,3),...
                'AutoScale','off','Color','k','LineWidth',2);
                r = atan2(q.VData,q.UData);
                degrees = r*180/pi;

                %diff_in_angle = (degrees(1,1) - degrees(2,1));
                diff_in_angle = min((360) - abs(degrees(1,1) - degrees(2,1)), abs(degrees(1,1) - degrees(2,1)));

                close(figgi)

                % initialize some variables
                stopID = 1;
                isnanu = NaN;

                % calculate global displacement
                Displacement = round(Second_arrow);
                newcenter2 = newcenter + Displacement;


                if ~isnan(distances3)

                Dist = distances2 + distances3;
                counter = 2;

                stopID = 1;
                isnanu = NaN;

                    while diff_in_angle < 90 & diff_in_angle > -90



                        [diff_in_angle, distances4, volume4, X4, dir4,stopID,isnanu,newcenter2] = nextarrow(newcenter2,radius_in_vox_xy,...
                        radius_in_vox_z,volSmooth_8bit,FinalPosMat3D,vSnorm,X2,voxelsize_xy);

                        if stopID == 0
                            break
                        end
                        if ~isnan(isnanu)
                            break
                        end

                        Dist = Dist + distances4;
                        counter = counter+1;
                        Vecs{counter+1,j} = X4;

                    end


                elseif isnan(distances3)
                    Dist = distances2;
                    counter = 1;
                else
                    Dist = 0; 
                    counter = 0;
                end

                if isnan(distances2) == 1
                    Dist_um(1,j) = NaN;
                    Dist_um(2,j) = 0;
                    Vecs{1,j} = Xstart;
                    Vecs{2,j} = 0;
                elseif isnan(distances3) == 1 & ~isnan(distances2)
                    Dist_um(1,j) = distances2*voxelsize_xy;
                    Dist_um(2,j) = 1;
                    Vecs{1,j} = Xstart;
                    Vecs{2,j} = X;
                    Vecs{2,j} = 0;
                else
                    Dist_um(1,j) = Dist*voxelsize_xy;
                    Dist_um(2,j) = counter;
                    Vecs{1,j} = Xstart;
                    Vecs{2,j} = X;
                    Vecs{3,j} = X2;
                end

                clearvars MaxCoord2 MaxCoord1 MaxCoordZ Max dmin dminind probeVolume_Dens probeVolume dmax...
                    Coord2min Coord2max Coord1min Coord1max zmin zmax Pix TwoDquery newPix HiInt centroids...
                    L MScoreInd score centroids volume distance volume2 distances2 q probeVolume_Dens2 Dist counter
            end 

            Dists_um{1,i} = Dist_um;
            Vecs_all{1,i} = Vecs;


            clearvars Coord2 Coord1 Coords Dist_um Vecs
        end

        
        
        
        
        d = 0;
        for i = 1:sep_z:size(FinalPosMat3D,3)
            Mind = Dists_um{1,i}(1,:) > 99999;
            Dists_um{1,i}(:,Mind) = [];
        end

        clearvars C
        for i = 1:sep_z:size(FinalPosMat3D,3)
            a = 1:sep_z:size(FinalPosMat3D,3);
            if i == 1
                C = cat(2,Dists_um{1,i}(1,:),Dists_um{1,i+sep_z}(1,:));
            end

            if i > 1 && i <= a(end)-sep_z
                C = cat(2,C,Dists_um{1,i+sep_z}(1,:));
            end

        end

        as = isnan(C);
        C(as) = [];
        
        % save C as the path lengths
        clearvars toexport toexport_c heading tosave
        toexport = [C'];
        toexport_c = num2cell(toexport);
        heading = {'Length of migration paths (um)'};
        tosave = [heading; toexport_c];
        xlswrite([newdir '\' 'MigrationPathsLength.xlsx'],tosave,1)
        
        % retrieve endpoint and save in .txt format for easy plotting in
        % Imaris using the spots from file XT and aalso as xlsx for
        % plotting elsewhere
        %clearvars EndPoint_array a b c d e Start
        %count = 0;
        %for i = 1:sep_z:size(FinalPosMat3D,3)
        %    a = Vecs_all{1,i};
        %    for j = 1:size(a,2)
        %        b = Vecs_all{1,i}(:,j);
        %        Start = Vecs_all{1,i}{1,j};
        %        for k = 1:size(b,1)
        %            c = Vecs_all{1,i}{k,j};
                    
        %            if ~isempty(c) & k ~= 1 & c ~= 0
        %                % the endpoint of the migration path
        %                d = c(1,:);    
        %                e = c(2,:);
        %                dist = d-e;
        %                Start = Start + dist;
        %                
        %            elseif c == 0
        %                Start = Start;
        %                break
        %                
        %            elseif isempty(c) & k ~= 1
        %                break  
        %            end
        %            
        %        end
        %        count = count+1;
        %        EndPoint_array(count,1:3) = Start;
        %    end
        %end
       % 
        %EndPoint_array(:,1) = EndPoint_array(:,1).*voxelsize_xy;
        %EndPoint_array(:,2) = EndPoint_array(:,2).*voxelsize_xy;
        %EndPoint_array(:,3) = EndPoint_array(:,3).*voxelsize_z;
        
        %EndPoint_array(:,1) = EndPoint_array(:,1) + str2double(get(handles.minx,'String'));
        %EndPoint_array(:,2) = EndPoint_array(:,2) + str2double(get(handles.miny,'String'));
        %EndPoint_array(:,3) = EndPoint_array(:,3) + str2double(get(handles.minz,'String'));
        
         % save EndPoint_array as the xlsx
        %clearvars toexport toexport_c heading tosave
        %toexport = [EndPoint_array];
        %toexport_c = num2cell(toexport);
        %heading = {'Position X' 'Position Y' 'Position Z'};
        %tosave = [heading; toexport_c];
        %xlswrite([newdir '\' 'Migration Endpoints.xlsx'],tosave,1)
        
        %radius = 0.3;
        %file = '\Migration path endpoints 1.txt';

        %fileID = fopen([newdir file],'w'); % text format used for CreateSpotsFromFile Xtension in Imaris
        %formatspec  = 'SpotRadius, %2d\n';
        %fprintf(fileID,formatspec,radius);
        %dlmwrite([newdir file],EndPoint_array,'-append','newline', 'pc', 'roffset',1)
        %fclose(fileID);
        %clearvars FileName2 FileName PathName

        %waitfor(msgbox('Export successful!'));
        drawnow;

end

%%
% save a log file with info about sigma and filtersize as well as pixelsize
% zslice size, dimensions etc.

if get(handles.radiobutton1,'value') == 1 
    
    what = {'size of filter xy (pixels)'; 'size of filter z (z slices)'; 'size of filter xy (um)'; 'size of filter z (um)';...
        'Sigma'; 'length of voxel (um)'; 'height of voxel (um)'; 'size in x (pixel)';...
        'size in x (um)'; 'size in y (pixel)'; 'size in y (um)'; 'size in z (pixel)'; 'size in z (um)';...
        'number of datapoints'; 'Heatmap adjustment applied'; 'Adjustment factor';'Hotspot threshold'; 'Manual threshold'; 'Manual threshold value'; 'Fission/Fusion thresholding';...
        'Change point thresholding'; 'Background saturation thresholding'};

    sett = {FilterSize; (zFilter); FilterSize_um; (zFilter*zslice); Sigma; pixelsize; zslice; size(volSmooth_8bit,1); size(volSmooth_8bit,1)*pixelsize;...
        size(volSmooth_8bit,2); size(volSmooth_8bit,2)*pixelsize; size(volSmooth_8bit,3); size(volSmooth_8bit,3)*zslice; size(XYZxyz,1); get(handles.radiobutton6,'Value');str2double(get(handles.adjfact,'String'));...
        hotspot_threshold; get(handles.radiobutton2,'Value'); str2double(get(handles.mantresh,'String'));get(handles.radiobutton3,'Value');...
        get(handles.radiobutton4,'Value');get(handles.radiobutton5,'Value')};
    
elseif get(handles.radiobutton1,'value') == 0 
    
    what = {'size of filter xy (pixels)'; 'size of filter z (z slices)'; 'size of filter xy (um)'; 'size of filter z (um)';...
        'Sigma'; 'length of voxel (um)'; 'height of voxel (um)'; 'size in x (pixel)';...
        'size in x (um)'; 'size in y (pixel)'; 'size in y (um)'; 'size in z (pixel)'; 'size in z (um)';...
        'number of datapoints'; 'Heatmap adjustment applied'; 'Adjustment factor'};

    sett = {FilterSize; (zFilter); FilterSize_um; (zFilter*zslice); Sigma; pixelsize; zslice; size(volSmooth_8bit,1); size(volSmooth_8bit,1)*pixelsize;...
        size(volSmooth_8bit,2); size(volSmooth_8bit,2)*pixelsize; size(volSmooth_8bit,3); size(volSmooth_8bit,3)*zslice; size(handles.XYZxyz,1);...
        get(handles.radiobutton6,'Value');str2double(get(handles.adjfact,'String'))};
    
end
toexport_last = [what sett];
xlswrite([newdir '\' 'Log.xlsx'],toexport_last,1)

set(DDD, 'pointer', 'arrow')
drawnow;


function zsize_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to zsize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of zsize_edittxt as text
%        str2double(get(hObject,'String')) returns contents of zsize_edittxt as a double


% --- Executes during object creation, after setting all properties.
function zsize_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to zsize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function psize_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to psize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of psize_edittxt as text
%        str2double(get(hObject,'String')) returns contents of psize_edittxt as a double


% --- Executes during object creation, after setting all properties.
function psize_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to psize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function z_size_Callback(hObject, eventdata, handles)
% hObject    handle to z_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of z_size as text
%        str2double(get(hObject,'String')) returns contents of z_size as a double


% --- Executes during object creation, after setting all properties.
function z_size_CreateFcn(hObject, eventdata, handles)
% hObject    handle to z_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in import_Btn.
function import_Btn_Callback(hObject, eventdata, handles)
% hObject    handle to import_Btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.zslices = str2double(get(handles.zsize_edittxt,'String'));
handles.handles.zslices = handles.zslices;


%% import position file
[filenamePOS, pathname] = ...
uigetfile({'*.csv';'*.*'}, 'Select XYZ position file, as exported from e.g. Imaris', 'MultiSelect', 'off');    
addpath(pathname)
    
%%
set(handles.tiff_txtBox,'String','Busy...')
drawnow;

%%
dataIm = importdata(filenamePOS);
aXYZIm = cell(size(5:(length(dataIm.textdata)),2),3);

for j = 1:3
    for k = 5:(length(dataIm.textdata))
        aXYZIm{k,j} = str2double(dataIm.textdata{k,j});
    end
end
XYZ = cell2mat(aXYZIm);

matrix = {zeros(str2double(get(handles.ysize_edittxt,'String')),str2double(get(handles.xsize_edittxt,'String')),str2double(get(handles.zsize_edittxt,'String')))};
FinalImage3D = matrix{1,1};

XYZxyz = PostoGrid(str2double(get(handles.minz,'String')),...
        str2double(get(handles.minx,'String')),...
        str2double(get(handles.miny,'String')),...
        str2double(get(handles.z_size,'String')),...
        str2double(get(handles.psize_edittxt,'String')),...
        str2double(get(handles.psize_edittxt,'String')),...
        matrix,XYZ);
    
% make matrix out of coordinates
for i = 1:size(XYZxyz,1)
    j = XYZxyz(i,5);
    k = XYZxyz(i,4);
    l = XYZxyz(i,6);
    FinalImage3D(j,k,l) = 255;
end

handles.FinalImages3D = (FinalImage3D);
handles.XYZxyz = XYZxyz;

set(handles.tiff_txtBox,'String',[filenamePOS])
drawnow

guidata(hObject, handles);



function sigma_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to sigma_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of sigma_edittxt as text
%        str2double(get(hObject,'String')) returns contents of sigma_edittxt as a double


% --- Executes during object creation, after setting all properties.
function sigma_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sigma_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function Filtersize_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to Filtersize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Filtersize_edittxt as text
%        str2double(get(hObject,'String')) returns contents of Filtersize_edittxt as a double


% --- Executes during object creation, after setting all properties.
function Filtersize_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Filtersize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in preview_Btn.
function preview_Btn_Callback(hObject, eventdata, handles)
% hObject    handle to preview_Btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
set(DDD, 'pointer', 'watch')
drawnow;

sigmas = get(handles.sigma_edittxt,'String'); % check if the user wants to see several sigmas (when commas are found)


Sigma = str2double(get(handles.sigma_edittxt,'String'));
FilterSize = str2double(get(handles.Filtersize_edittxt,'String'));

% import the rectangle corners previsously chosen
nodes = handles.nodes;

% go through the stack, find a a good slice
FinalImages3D_orig = handles.FinalImages3D;%{1,1};

% for further processing use only choosen ROI
FinalImages3D = FinalImages3D_orig(nodes(1,2):nodes(3,2),nodes(1,1):nodes(2,1),:);

Sums = zeros(1,size(FinalImages3D,3));
for i = 1:size(FinalImages3D,3)
    imageofmask = FinalImages3D(:,:,i);
    Sums(1,i) = sum(sum(imageofmask));
end
[~,bestslice,~] = intersect(Sums,max(Sums));

slice = FinalImages3D(:,:,bestslice);

if bestslice - 20 > 0 & bestslice + 20 <= size(FinalImages3D,3)
    middle = 21;
    smallstack = FinalImages3D(:,:,bestslice-20:bestslice+20);
elseif bestslice - 10 > 0 & bestslice + 10 <= size(FinalImages3D,3)
    middle = 11;
    smallstack = FinalImages3D(:,:,bestslice-10:bestslice+10);
elseif bestslice - 5 > 0 & bestslice + 5 <= size(FinalImages3D,3)
    middle = 6;
    smallstack = FinalImages3D(:,:,bestslice-5:bestslice+5);
elseif bestslice - 2 > 0 & bestslice + 2 <= size(FinalImages3D,3)
    middle = 3;
    smallstack = FinalImages3D(:,:,bestslice-2:bestslice+2);
elseif bestslice - 1 > 0 & bestslice + 1 <= size(FinalImages3D,3)
    middle = 1;
    smallstack = FinalImages3D(:,:,bestslice-1:bestslice+1);
elseif bestslice - 1 == 0
    middle = 2;
    smallstack = FinalImages3D(:,:,bestslice:bestslice+3);
end

ind = find(smallstack(:,:,middle) >= 10);

FilterSize_um = str2double(get(handles.Filtersize_edittxt,'String'));
pixelsize = str2double(get(handles.psize_edittxt,'String'));
zslice = (str2double(get(handles.z_size,'String')));

FilterSize = 2*floor((FilterSize_um/pixelsize)/2)-1;

pre_zFilter = round(FilterSize/(zslice/pixelsize));
zFilter = 2*floor(pre_zFilter/2)-1;

%%

Sigmaz = (Sigma/(zslice/pixelsize));
volSmooth = imgaussfilt3(smallstack, [Sigma Sigma Sigmaz],...
    'FilterSize', [FilterSize FilterSize zFilter],...
    'FilterDomain','frequency',...    
    'Padding',0);
    %'FilterDomain','frequency',...
    %'Padding','circular');
    
Maxi = max(max(max((volSmooth(:,:,:)))));
vSnorm = volSmooth./Maxi;
aq = find(vSnorm < 0);
vSnorm(aq) = 0;

vSnorm_8bit = vSnorm*255;
volSmooth_8bit = uint8(vSnorm_8bit);

handles.volSmooth = volSmooth;
handles.volSmooth_8bit = volSmooth_8bit;
handles.smallstack = smallstack;
%final = FinalImages3D(:,:,bestslice) >= 255;
volSmooth_8bit2D = volSmooth_8bit(:,:,middle);
volSmooth_8bit2D(ind) = 256;

handles.middle = middle;
handles.ind = ind;

set(handles.text8,'String',['You are looking at section: ' num2str(middle-1) ' of ' num2str(get(handles.zslider,'Max')) ' sections.'])
drawnow
set(handles.zslider,'Max',(2*(middle-1)));
set(handles.zslider,'Min',1);
set(handles.zslider,'Value',middle);

axes(handles.axes1)
image(volSmooth_8bit2D)
xticklabels([])
yticklabels([])
myColorMap = jet(256); 
myColorMap(256, :) = [1 1 1];
colormap(myColorMap);
set(DDD,'toolbar','figure');
%volSmooth = imgaussfilt3(FinalImages3D, Sigma, 'FilterSize', FilterSize);
%volSmooth_x = volSmooth > 8;

set(DDD, 'pointer', 'arrow')
drawnow;
guidata(hObject, handles);



% --- Executes on button press in knn_Btn.
function knn_Btn_Callback(hObject, eventdata, handles)
% hObject    handle to knn_Btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)



% --- Executes on slider movement.
function zslider_Callback(hObject, eventdata, handles)
% hObject    handle to zslider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
slicedec = get(handles.zslider,'Value');
slice = round(slicedec);

volSmooth_8bit = handles.volSmooth_8bit;


smallstack = handles.smallstack;

ind = find(smallstack(:,:,slice) >= 10);
volSmooth_8bit2D = volSmooth_8bit(:,:,slice);
volSmooth_8bit2D(ind) = 256;

axes(handles.axes1)
image(volSmooth_8bit2D)
xticklabels([])
yticklabels([])
myColorMap = jet(256); 
myColorMap(256, :) = [1 1 1];
colormap(myColorMap);

set(handles.text8,'String',['You are looking at section: ' num2str(slice) ' of ' num2str(get(handles.zslider,'Max')) ' sections.'])
drawnow

% --- Executes during object creation, after setting all properties.
function zslider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to zslider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on button press in chprar_btn.
function chprar_btn_Callback(hObject, eventdata, handles)
% hObject    handle to chprar_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
FinalImages3D = handles.FinalImages3D;%{1,1};
MIP = max(FinalImages3D,[],3);

fig = figure();
ax = gca;
imshow(MIP)

h = imrect(ax);
setColor(h,'r');
nodes1 = getPosition(h);

close(fig)

set(DDD, 'pointer', 'watch')
drawnow;

leftdown = nodes1(1,1:2);
rightdown = [(nodes1(1,1) + nodes1(1,3)), nodes1(1,2)];
rightup = [(nodes1(1,1) + nodes1(1,3)), (nodes1(1,2) + nodes1(1,4))];
leftup = [nodes1(1,1), (nodes1(1,2) + nodes1(1,4))];
nodes = [round(leftdown); round(rightdown); round(rightup); round(leftup)];

handles.nodes = nodes;
set(DDD, 'pointer', 'arrow')
drawnow;
guidata(hObject, handles);


% --- Executes on button press in radiobutton1.
function radiobutton1_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton1



function minz_Callback(hObject, eventdata, handles)
% hObject    handle to minz (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of minz as text
%        str2double(get(hObject,'String')) returns contents of minz as a double


% --- Executes during object creation, after setting all properties.
function minz_CreateFcn(hObject, eventdata, handles)
% hObject    handle to minz (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function minx_Callback(hObject, eventdata, handles)
% hObject    handle to minx (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of minx as text
%        str2double(get(hObject,'String')) returns contents of minx as a double


% --- Executes during object creation, after setting all properties.
function minx_CreateFcn(hObject, eventdata, handles)
% hObject    handle to minx (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function miny_Callback(hObject, eventdata, handles)
% hObject    handle to miny (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of miny as text
%        str2double(get(hObject,'String')) returns contents of miny as a double


% --- Executes during object creation, after setting all properties.
function miny_CreateFcn(hObject, eventdata, handles)
% hObject    handle to miny (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function zsilce_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to zsilce_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of zsilce_edittxt as text
%        str2double(get(hObject,'String')) returns contents of zsilce_edittxt as a double


% --- Executes during object creation, after setting all properties.
function zsilce_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to zsilce_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function xsize_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to xsize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of xsize_edittxt as text
%        str2double(get(hObject,'String')) returns contents of xsize_edittxt as a double


% --- Executes during object creation, after setting all properties.
function xsize_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to xsize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function ysize_edittxt_Callback(hObject, eventdata, handles)
% hObject    handle to ysize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ysize_edittxt as text
%        str2double(get(hObject,'String')) returns contents of ysize_edittxt as a double


% --- Executes during object creation, after setting all properties.
function ysize_edittxt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ysize_edittxt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function randnumb_Callback(hObject, eventdata, handles)
% hObject    handle to randnumb (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of randnumb as text
%        str2double(get(hObject,'String')) returns contents of randnumb as a double


% --- Executes during object creation, after setting all properties.
function randnumb_CreateFcn(hObject, eventdata, handles)
% hObject    handle to randnumb (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on key press with focus on randnumb and none of its controls.
function randnumb_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to randnumb (see GCBO)
% eventdata  structure with the following fields (see MATLAB.UI.CONTROL.UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in calc_fromHM.
function calc_fromHM_Callback(hObject, eventdata, handles)
% hObject    handle to calc_fromHM (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)



% --- Executes on button press in radiobutton2.
function radiobutton2_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton2
if get(handles.radiobutton2,'Value') == 1
    set(handles.radiobutton3,'Value',0)
    set(handles.radiobutton4,'Value',0)
    set(handles.radiobutton5,'Value',0)

else

end


function mantresh_Callback(hObject, eventdata, handles)
% hObject    handle to mantresh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of mantresh as text
%        str2double(get(hObject,'String')) returns contents of mantresh as a double


% --- Executes during object creation, after setting all properties.
function mantresh_CreateFcn(hObject, eventdata, handles)
% hObject    handle to mantresh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radiobutton3.
function radiobutton3_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton3
if get(handles.radiobutton3,'Value') == 1
    set(handles.radiobutton2,'Value',0)
    set(handles.radiobutton4,'Value',0)
    set(handles.radiobutton5,'Value',0)
else

end

% --- Executes on button press in radiobutton4.
function radiobutton4_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton4
if get(handles.radiobutton4,'Value') == 1
    set(handles.radiobutton2,'Value',0)
    set(handles.radiobutton3,'Value',0)
    set(handles.radiobutton5,'Value',0)
else

end

% --- Executes on button press in radiobutton5.
function radiobutton5_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton5
if get(handles.radiobutton5,'Value') == 1
    set(handles.radiobutton2,'Value',0)
    set(handles.radiobutton3,'Value',0)
    set(handles.radiobutton4,'Value',0)
else

end


function adjfact_Callback(hObject, eventdata, handles)
% hObject    handle to adjfact (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of adjfact as text
%        str2double(get(hObject,'String')) returns contents of adjfact as a double


% --- Executes during object creation, after setting all properties.
function adjfact_CreateFcn(hObject, eventdata, handles)
% hObject    handle to adjfact (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in applyadj.
function applyadj_Callback(hObject, eventdata, handles)
% hObject    handle to applyadj (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
adjfact = str2double(get(handles.adjfact,'String'));
    
volSmooth = handles.volSmooth;
middle = handles.middle;
ind = handles.ind;

vSnorm = volSmooth./adjfact;
aq = find(vSnorm < 0);
vSnorm(aq) = 0;

vSnorm_8bit = vSnorm*255;
volSmooth_8bit = uint8(vSnorm_8bit);

volSmooth_8bit2D = volSmooth_8bit(:,:,middle);
volSmooth_8bit2D(ind) = 256;

set(handles.text8,'String',['You are looking at section: ' num2str(middle-1) ' of ' num2str(get(handles.zslider,'Max')) ' sections.'])
drawnow
set(handles.zslider,'Max',(2*(middle-1)));
set(handles.zslider,'Min',1);
set(handles.zslider,'Value',middle);

axes(handles.axes1)
image(volSmooth_8bit2D)
xticklabels([])
yticklabels([])
myColorMap = jet(256); 
myColorMap(256, :) = [1 1 1];
colormap(myColorMap);
set(DDD,'toolbar','figure');

if get(handles.radiobutton6,'Value') == 1
    handles.volSmooth_8bit = volSmooth_8bit;
end

set(DDD, 'pointer', 'arrow')
drawnow;
guidata(hObject, handles);

% --- Executes on button press in radiobutton6.
function radiobutton6_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton6



function sizethresh_Callback(hObject, eventdata, handles)
% hObject    handle to sizethresh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of sizethresh as text
%        str2double(get(hObject,'String')) returns contents of sizethresh as a double


% --- Executes during object creation, after setting all properties.
function sizethresh_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sizethresh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radiobutton8.
function radiobutton8_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton8 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton8


% --- Executes on button press in radiobutton9.
function radiobutton9_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radiobutton9



function radius_txt_Callback(hObject, eventdata, handles)
% hObject    handle to radius_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of radius_txt as text
%        str2double(get(hObject,'String')) returns contents of radius_txt as a double


% --- Executes during object creation, after setting all properties.
function radius_txt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to radius_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function spacing_txt_Callback(hObject, eventdata, handles)
% hObject    handle to spacing_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of spacing_txt as text
%        str2double(get(hObject,'String')) returns contents of spacing_txt as a double


% --- Executes during object creation, after setting all properties.
function spacing_txt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to spacing_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function spacingZ_txt_Callback(hObject, eventdata, handles)
% hObject    handle to spacingZ_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of spacingZ_txt as text
%        str2double(get(hObject,'String')) returns contents of spacingZ_txt as a double


% --- Executes during object creation, after setting all properties.
function spacingZ_txt_CreateFcn(hObject, eventdata, handles)
% hObject    handle to spacingZ_txt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
